# -*- coding: utf-8 -*-
# Copyright 2022 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
import proto  # type: ignore

from google.ads.googleads.v11.common.types import criteria
from google.ads.googleads.v11.enums.types import criterion_type


__protobuf__ = proto.module(
    package="google.ads.googleads.v11.resources",
    marshal="google.ads.googleads.v11",
    manifest={"CustomerNegativeCriterion",},
)


class CustomerNegativeCriterion(proto.Message):
    r"""A negative criterion for exclusions at the customer level.

    This message has `oneof`_ fields (mutually exclusive fields).
    For each oneof, at most one member field can be set at the same time.
    Setting any member of the oneof automatically clears all other
    members.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        resource_name (str):
            Immutable. The resource name of the customer negative
            criterion. Customer negative criterion resource names have
            the form:

            ``customers/{customer_id}/customerNegativeCriteria/{criterion_id}``
        id (int):
            Output only. The ID of the criterion.

            This field is a member of `oneof`_ ``_id``.
        type_ (google.ads.googleads.v11.enums.types.CriterionTypeEnum.CriterionType):
            Output only. The type of the criterion.
        content_label (google.ads.googleads.v11.common.types.ContentLabelInfo):
            Immutable. ContentLabel.

            This field is a member of `oneof`_ ``criterion``.
        mobile_application (google.ads.googleads.v11.common.types.MobileApplicationInfo):
            Immutable. MobileApplication.

            This field is a member of `oneof`_ ``criterion``.
        mobile_app_category (google.ads.googleads.v11.common.types.MobileAppCategoryInfo):
            Immutable. MobileAppCategory.

            This field is a member of `oneof`_ ``criterion``.
        placement (google.ads.googleads.v11.common.types.PlacementInfo):
            Immutable. Placement.

            This field is a member of `oneof`_ ``criterion``.
        youtube_video (google.ads.googleads.v11.common.types.YouTubeVideoInfo):
            Immutable. YouTube Video.

            This field is a member of `oneof`_ ``criterion``.
        youtube_channel (google.ads.googleads.v11.common.types.YouTubeChannelInfo):
            Immutable. YouTube Channel.

            This field is a member of `oneof`_ ``criterion``.
    """

    resource_name = proto.Field(proto.STRING, number=1,)
    id = proto.Field(proto.INT64, number=10, optional=True,)
    type_ = proto.Field(
        proto.ENUM,
        number=3,
        enum=criterion_type.CriterionTypeEnum.CriterionType,
    )
    content_label = proto.Field(
        proto.MESSAGE,
        number=4,
        oneof="criterion",
        message=criteria.ContentLabelInfo,
    )
    mobile_application = proto.Field(
        proto.MESSAGE,
        number=5,
        oneof="criterion",
        message=criteria.MobileApplicationInfo,
    )
    mobile_app_category = proto.Field(
        proto.MESSAGE,
        number=6,
        oneof="criterion",
        message=criteria.MobileAppCategoryInfo,
    )
    placement = proto.Field(
        proto.MESSAGE,
        number=7,
        oneof="criterion",
        message=criteria.PlacementInfo,
    )
    youtube_video = proto.Field(
        proto.MESSAGE,
        number=8,
        oneof="criterion",
        message=criteria.YouTubeVideoInfo,
    )
    youtube_channel = proto.Field(
        proto.MESSAGE,
        number=9,
        oneof="criterion",
        message=criteria.YouTubeChannelInfo,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2022 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v10.resources",
    marshal="google.ads.googleads.v10",
    manifest={
        "SmartCampaignSetting",
    },
)


class SmartCampaignSetting(proto.Message):
    r"""Settings for configuring Smart campaigns.

    This message has `oneof`_ fields (mutually exclusive fields).
    For each oneof, at most one member field can be set at the same time.
    Setting any member of the oneof automatically clears all other
    members.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        resource_name (str):
            Immutable. The resource name of the Smart campaign setting.
            Smart campaign setting resource names have the form:

            ``customers/{customer_id}/smartCampaignSettings/{campaign_id}``
        campaign (str):
            Output only. The campaign to which these
            settings apply.
        phone_number (google.ads.googleads.v10.resources.types.SmartCampaignSetting.PhoneNumber):
            Phone number and country code.
        final_url (str):
            Landing page url given by user for this
            Campaign.
        advertising_language_code (str):
            The ISO-639-1 language code to advertise in.
        business_name (str):
            The name of the business.

            This field is a member of `oneof`_ ``business_setting``.
        business_location_id (int):
            The ID of the Business Profile location. The location ID can
            be fetched by Business Profile API with its form:
            accounts/{accountId}/locations/{locationId}. The last
            {locationId} component from the Business Profile API
            represents the business_location_id. See the [Business
            Profile API]
            (https://developers.google.com/my-business/reference/rest/v4/accounts.locations)

            This field is a member of `oneof`_ ``business_setting``.
    """

    class PhoneNumber(proto.Message):
        r"""Phone number and country code in smart campaign settings.

        Attributes:
            phone_number (str):
                Phone number of the smart campaign.

                This field is a member of `oneof`_ ``_phone_number``.
            country_code (str):
                Upper-case, two-letter country code as
                defined by ISO-3166.

                This field is a member of `oneof`_ ``_country_code``.
        """

        phone_number = proto.Field(
            proto.STRING,
            number=1,
            optional=True,
        )
        country_code = proto.Field(
            proto.STRING,
            number=2,
            optional=True,
        )

    resource_name = proto.Field(
        proto.STRING,
        number=1,
    )
    campaign = proto.Field(
        proto.STRING,
        number=2,
    )
    phone_number = proto.Field(
        proto.MESSAGE,
        number=3,
        message=PhoneNumber,
    )
    final_url = proto.Field(
        proto.STRING,
        number=4,
    )
    advertising_language_code = proto.Field(
        proto.STRING,
        number=7,
    )
    business_name = proto.Field(
        proto.STRING,
        number=5,
        oneof="business_setting",
    )
    business_location_id = proto.Field(
        proto.INT64,
        number=6,
        oneof="business_setting",
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2022 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
import proto  # type: ignore

from google.ads.googleads.v10.common.types import customizer_value
from google.ads.googleads.v10.enums.types import customizer_value_status


__protobuf__ = proto.module(
    package="google.ads.googleads.v10.resources",
    marshal="google.ads.googleads.v10",
    manifest={
        "CustomerCustomizer",
    },
)


class CustomerCustomizer(proto.Message):
    r"""A customizer value for the associated CustomizerAttribute at
    the Customer level.

    Attributes:
        resource_name (str):
            Immutable. The resource name of the customer customizer.
            Customer customizer resource names have the form:

            ``customers/{customer_id}/customerCustomizers/{customizer_attribute_id}``
        customizer_attribute (str):
            Required. Immutable. The customizer attribute
            which is linked to the customer.
        status (google.ads.googleads.v10.enums.types.CustomizerValueStatusEnum.CustomizerValueStatus):
            Output only. The status of the customer
            customizer attribute.
        value (google.ads.googleads.v10.common.types.CustomizerValue):
            Required. The value to associate with the
            customizer attribute at this level. The value
            must be of the type specified for the
            CustomizerAttribute.
    """

    resource_name = proto.Field(
        proto.STRING,
        number=1,
    )
    customizer_attribute = proto.Field(
        proto.STRING,
        number=2,
    )
    status = proto.Field(
        proto.ENUM,
        number=3,
        enum=customizer_value_status.CustomizerValueStatusEnum.CustomizerValueStatus,
    )
    value = proto.Field(
        proto.MESSAGE,
        number=4,
        message=customizer_value.CustomizerValue,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2022 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
import proto  # type: ignore

from google.ads.googleads.v10.common.types import custom_parameter
from google.ads.googleads.v10.common.types import (
    explorer_auto_optimizer_setting as gagc_explorer_auto_optimizer_setting,
)
from google.ads.googleads.v10.common.types import (
    targeting_setting as gagc_targeting_setting,
)
from google.ads.googleads.v10.enums.types import ad_group_ad_rotation_mode
from google.ads.googleads.v10.enums.types import ad_group_status
from google.ads.googleads.v10.enums.types import ad_group_type
from google.ads.googleads.v10.enums.types import asset_field_type
from google.ads.googleads.v10.enums.types import bidding_source
from google.ads.googleads.v10.enums.types import targeting_dimension


__protobuf__ = proto.module(
    package="google.ads.googleads.v10.resources",
    marshal="google.ads.googleads.v10",
    manifest={
        "AdGroup",
    },
)


class AdGroup(proto.Message):
    r"""An ad group.

    Attributes:
        resource_name (str):
            Immutable. The resource name of the ad group. Ad group
            resource names have the form:

            ``customers/{customer_id}/adGroups/{ad_group_id}``
        id (int):
            Output only. The ID of the ad group.

            This field is a member of `oneof`_ ``_id``.
        name (str):
            The name of the ad group.
            This field is required and should not be empty
            when creating new ad groups.

            It must contain fewer than 255 UTF-8 full-width
            characters.
            It must not contain any null (code point 0x0),
            NL line feed (code point 0xA) or carriage return
            (code point 0xD) characters.

            This field is a member of `oneof`_ ``_name``.
        status (google.ads.googleads.v10.enums.types.AdGroupStatusEnum.AdGroupStatus):
            The status of the ad group.
        type_ (google.ads.googleads.v10.enums.types.AdGroupTypeEnum.AdGroupType):
            Immutable. The type of the ad group.
        ad_rotation_mode (google.ads.googleads.v10.enums.types.AdGroupAdRotationModeEnum.AdGroupAdRotationMode):
            The ad rotation mode of the ad group.
        base_ad_group (str):
            Output only. For draft or experiment ad
            groups, this field is the resource name of the
            base ad group from which this ad group was
            created. If a draft or experiment ad group does
            not have a base ad group, then this field is
            null.
            For base ad groups, this field equals the ad
            group resource name.
            This field is read-only.

            This field is a member of `oneof`_ ``_base_ad_group``.
        tracking_url_template (str):
            The URL template for constructing a tracking
            URL.

            This field is a member of `oneof`_ ``_tracking_url_template``.
        url_custom_parameters (Sequence[google.ads.googleads.v10.common.types.CustomParameter]):
            The list of mappings used to substitute custom parameter
            tags in a ``tracking_url_template``, ``final_urls``, or
            ``mobile_final_urls``.
        campaign (str):
            Immutable. The campaign to which the ad group
            belongs.

            This field is a member of `oneof`_ ``_campaign``.
        cpc_bid_micros (int):
            The maximum CPC (cost-per-click) bid.

            This field is a member of `oneof`_ ``_cpc_bid_micros``.
        effective_cpc_bid_micros (int):
            Output only. Value will be same as that of
            the CPC (cost-per-click) bid value when the
            bidding strategy is one of manual cpc, enhanced
            cpc, page one promoted or target outrank share,
            otherwise the value will be null.

            This field is a member of `oneof`_ ``_effective_cpc_bid_micros``.
        cpm_bid_micros (int):
            The maximum CPM (cost-per-thousand viewable
            impressions) bid.

            This field is a member of `oneof`_ ``_cpm_bid_micros``.
        target_cpa_micros (int):
            The target CPA (cost-per-acquisition). If the ad group's
            campaign bidding strategy is TargetCpa or
            MaximizeConversions (with its target_cpa field set), then
            this field overrides the target CPA specified in the
            campaign's bidding strategy. Otherwise, this value is
            ignored.

            This field is a member of `oneof`_ ``_target_cpa_micros``.
        cpv_bid_micros (int):
            Output only. The CPV (cost-per-view) bid.

            This field is a member of `oneof`_ ``_cpv_bid_micros``.
        target_cpm_micros (int):
            Average amount in micros that the advertiser
            is willing to pay for every thousand times the
            ad is shown.

            This field is a member of `oneof`_ ``_target_cpm_micros``.
        target_roas (float):
            The target ROAS (return-on-ad-spend) override. If the ad
            group's campaign bidding strategy is TargetRoas or
            MaximizeConversionValue (with its target_roas field set),
            then this field overrides the target ROAS specified in the
            campaign's bidding strategy. Otherwise, this value is
            ignored.

            This field is a member of `oneof`_ ``_target_roas``.
        percent_cpc_bid_micros (int):
            The percent cpc bid amount, expressed as a fraction of the
            advertised price for some good or service. The valid range
            for the fraction is [0,1) and the value stored here is
            1,000,000 \* [fraction].

            This field is a member of `oneof`_ ``_percent_cpc_bid_micros``.
        explorer_auto_optimizer_setting (google.ads.googleads.v10.common.types.ExplorerAutoOptimizerSetting):
            Settings for the Display Campaign Optimizer,
            initially termed "Explorer".
        display_custom_bid_dimension (google.ads.googleads.v10.enums.types.TargetingDimensionEnum.TargetingDimension):
            Allows advertisers to specify a targeting
            dimension on which to place absolute bids. This
            is only applicable for campaigns that target
            only the display network and not search.
        final_url_suffix (str):
            URL template for appending params to Final
            URL.

            This field is a member of `oneof`_ ``_final_url_suffix``.
        targeting_setting (google.ads.googleads.v10.common.types.TargetingSetting):
            Setting for targeting related features.
        audience_setting (google.ads.googleads.v10.resources.types.AdGroup.AudienceSetting):
            Immutable. Setting for audience related
            features.
        effective_target_cpa_micros (int):
            Output only. The effective target CPA
            (cost-per-acquisition). This field is read-only.

            This field is a member of `oneof`_ ``_effective_target_cpa_micros``.
        effective_target_cpa_source (google.ads.googleads.v10.enums.types.BiddingSourceEnum.BiddingSource):
            Output only. Source of the effective target
            CPA. This field is read-only.
        effective_target_roas (float):
            Output only. The effective target ROAS
            (return-on-ad-spend). This field is read-only.

            This field is a member of `oneof`_ ``_effective_target_roas``.
        effective_target_roas_source (google.ads.googleads.v10.enums.types.BiddingSourceEnum.BiddingSource):
            Output only. Source of the effective target
            ROAS. This field is read-only.
        labels (Sequence[str]):
            Output only. The resource names of labels
            attached to this ad group.
        excluded_parent_asset_field_types (Sequence[google.ads.googleads.v10.enums.types.AssetFieldTypeEnum.AssetFieldType]):
            The asset field types that should be excluded
            from this ad group. Asset links with these field
            types will not be inherited by this ad group
            from the upper levels.
    """

    class AudienceSetting(proto.Message):
        r"""Settings for the audience targeting.

        Attributes:
            use_audience_grouped (bool):
                Immutable. If true, this ad group uses an
                Audience resource for audience targeting. If
                false, this ad group may use audience segment
                criteria instead.
        """

        use_audience_grouped = proto.Field(
            proto.BOOL,
            number=1,
        )

    resource_name = proto.Field(
        proto.STRING,
        number=1,
    )
    id = proto.Field(
        proto.INT64,
        number=34,
        optional=True,
    )
    name = proto.Field(
        proto.STRING,
        number=35,
        optional=True,
    )
    status = proto.Field(
        proto.ENUM,
        number=5,
        enum=ad_group_status.AdGroupStatusEnum.AdGroupStatus,
    )
    type_ = proto.Field(
        proto.ENUM,
        number=12,
        enum=ad_group_type.AdGroupTypeEnum.AdGroupType,
    )
    ad_rotation_mode = proto.Field(
        proto.ENUM,
        number=22,
        enum=ad_group_ad_rotation_mode.AdGroupAdRotationModeEnum.AdGroupAdRotationMode,
    )
    base_ad_group = proto.Field(
        proto.STRING,
        number=36,
        optional=True,
    )
    tracking_url_template = proto.Field(
        proto.STRING,
        number=37,
        optional=True,
    )
    url_custom_parameters = proto.RepeatedField(
        proto.MESSAGE,
        number=6,
        message=custom_parameter.CustomParameter,
    )
    campaign = proto.Field(
        proto.STRING,
        number=38,
        optional=True,
    )
    cpc_bid_micros = proto.Field(
        proto.INT64,
        number=39,
        optional=True,
    )
    effective_cpc_bid_micros = proto.Field(
        proto.INT64,
        number=57,
        optional=True,
    )
    cpm_bid_micros = proto.Field(
        proto.INT64,
        number=40,
        optional=True,
    )
    target_cpa_micros = proto.Field(
        proto.INT64,
        number=41,
        optional=True,
    )
    cpv_bid_micros = proto.Field(
        proto.INT64,
        number=42,
        optional=True,
    )
    target_cpm_micros = proto.Field(
        proto.INT64,
        number=43,
        optional=True,
    )
    target_roas = proto.Field(
        proto.DOUBLE,
        number=44,
        optional=True,
    )
    percent_cpc_bid_micros = proto.Field(
        proto.INT64,
        number=45,
        optional=True,
    )
    explorer_auto_optimizer_setting = proto.Field(
        proto.MESSAGE,
        number=21,
        message=gagc_explorer_auto_optimizer_setting.ExplorerAutoOptimizerSetting,
    )
    display_custom_bid_dimension = proto.Field(
        proto.ENUM,
        number=23,
        enum=targeting_dimension.TargetingDimensionEnum.TargetingDimension,
    )
    final_url_suffix = proto.Field(
        proto.STRING,
        number=46,
        optional=True,
    )
    targeting_setting = proto.Field(
        proto.MESSAGE,
        number=25,
        message=gagc_targeting_setting.TargetingSetting,
    )
    audience_setting = proto.Field(
        proto.MESSAGE,
        number=56,
        message=AudienceSetting,
    )
    effective_target_cpa_micros = proto.Field(
        proto.INT64,
        number=47,
        optional=True,
    )
    effective_target_cpa_source = proto.Field(
        proto.ENUM,
        number=29,
        enum=bidding_source.BiddingSourceEnum.BiddingSource,
    )
    effective_target_roas = proto.Field(
        proto.DOUBLE,
        number=48,
        optional=True,
    )
    effective_target_roas_source = proto.Field(
        proto.ENUM,
        number=32,
        enum=bidding_source.BiddingSourceEnum.BiddingSource,
    )
    labels = proto.RepeatedField(
        proto.STRING,
        number=49,
    )
    excluded_parent_asset_field_types = proto.RepeatedField(
        proto.ENUM,
        number=54,
        enum=asset_field_type.AssetFieldTypeEnum.AssetFieldType,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2022 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from collections import OrderedDict
import os
import re
from typing import Dict, Optional, Sequence, Tuple, Type, Union
import pkg_resources

from google.api_core import client_options as client_options_lib
from google.api_core import gapic_v1
from google.api_core import retry as retries
from google.auth import credentials as ga_credentials  # type: ignore
from google.auth.transport import mtls  # type: ignore
from google.auth.transport.grpc import SslCredentials  # type: ignore
from google.auth.exceptions import MutualTLSChannelError  # type: ignore
from google.oauth2 import service_account  # type: ignore

try:
    OptionalRetry = Union[retries.Retry, gapic_v1.method._MethodDefault]
except AttributeError:  # pragma: NO COVER
    OptionalRetry = Union[retries.Retry, object]  # type: ignore

from google.ads.googleads.v11.services.services.campaign_draft_service import (
    pagers,
)
from google.ads.googleads.v11.services.types import campaign_draft_service
from google.api_core import operation  # type: ignore
from google.api_core import operation_async  # type: ignore
from google.protobuf import empty_pb2  # type: ignore
from google.rpc import status_pb2  # type: ignore
from .transports.base import CampaignDraftServiceTransport, DEFAULT_CLIENT_INFO
from .transports.grpc import CampaignDraftServiceGrpcTransport


class CampaignDraftServiceClientMeta(type):
    """Metaclass for the CampaignDraftService client.

    This provides class-level methods for building and retrieving
    support objects (e.g. transport) without polluting the client instance
    objects.
    """

    _transport_registry = (
        OrderedDict()
    )  # type: Dict[str, Type[CampaignDraftServiceTransport]]
    _transport_registry["grpc"] = CampaignDraftServiceGrpcTransport

    def get_transport_class(
        cls, label: str = None,
    ) -> Type[CampaignDraftServiceTransport]:
        """Returns an appropriate transport class.

        Args:
            label: The name of the desired transport. If none is
                provided, then the first transport in the registry is used.

        Returns:
            The transport class to use.
        """
        # If a specific transport is requested, return that one.
        if label:
            return cls._transport_registry[label]

        # No transport is requested; return the default (that is, the first one
        # in the dictionary).
        return next(iter(cls._transport_registry.values()))


class CampaignDraftServiceClient(metaclass=CampaignDraftServiceClientMeta):
    """Service to manage campaign drafts."""

    @staticmethod
    def _get_default_mtls_endpoint(api_endpoint):
        """Converts api endpoint to mTLS endpoint.

        Convert "*.sandbox.googleapis.com" and "*.googleapis.com" to
        "*.mtls.sandbox.googleapis.com" and "*.mtls.googleapis.com" respectively.
        Args:
            api_endpoint (Optional[str]): the api endpoint to convert.
        Returns:
            str: converted mTLS api endpoint.
        """
        if not api_endpoint:
            return api_endpoint

        mtls_endpoint_re = re.compile(
            r"(?P<name>[^.]+)(?P<mtls>\.mtls)?(?P<sandbox>\.sandbox)?(?P<googledomain>\.googleapis\.com)?"
        )

        m = mtls_endpoint_re.match(api_endpoint)
        name, mtls, sandbox, googledomain = m.groups()
        if mtls or not googledomain:
            return api_endpoint

        if sandbox:
            return api_endpoint.replace(
                "sandbox.googleapis.com", "mtls.sandbox.googleapis.com"
            )

        return api_endpoint.replace(".googleapis.com", ".mtls.googleapis.com")

    DEFAULT_ENDPOINT = "googleads.googleapis.com"
    DEFAULT_MTLS_ENDPOINT = _get_default_mtls_endpoint.__func__(  # type: ignore
        DEFAULT_ENDPOINT
    )

    @classmethod
    def from_service_account_info(cls, info: dict, *args, **kwargs):
        """Creates an instance of this client using the provided credentials
            info.

        Args:
            info (dict): The service account private key info.
            args: Additional arguments to pass to the constructor.
            kwargs: Additional arguments to pass to the constructor.

        Returns:
            CampaignDraftServiceClient: The constructed client.
        """
        credentials = service_account.Credentials.from_service_account_info(
            info
        )
        kwargs["credentials"] = credentials
        return cls(*args, **kwargs)

    @classmethod
    def from_service_account_file(cls, filename: str, *args, **kwargs):
        """Creates an instance of this client using the provided credentials
        file.

        Args:
            filename (str): The path to the service account private key json
                file.
            args: Additional arguments to pass to the constructor.
            kwargs: Additional arguments to pass to the constructor.

        Returns:
            CampaignDraftServiceClient: The constructed client.
        """
        credentials = service_account.Credentials.from_service_account_file(
            filename
        )
        kwargs["credentials"] = credentials
        return cls(*args, **kwargs)

    from_service_account_json = from_service_account_file

    @property
    def transport(self) -> CampaignDraftServiceTransport:
        """Returns the transport used by the client instance.

        Returns:
            CampaignDraftServiceTransport: The transport used by the client
                instance.
        """
        return self._transport

    def __enter__(self):
        return self

    def __exit__(self, type, value, traceback):
        """Releases underlying transport's resources.

        .. warning::
            ONLY use as a context manager if the transport is NOT shared
            with other clients! Exiting the with block will CLOSE the transport
            and may cause errors in other clients!
        """
        self.transport.close()

    @staticmethod
    def campaign_path(customer_id: str, campaign_id: str,) -> str:
        """Returns a fully-qualified campaign string."""
        return "customers/{customer_id}/campaigns/{campaign_id}".format(
            customer_id=customer_id, campaign_id=campaign_id,
        )

    @staticmethod
    def parse_campaign_path(path: str) -> Dict[str, str]:
        """Parses a campaign path into its component segments."""
        m = re.match(
            r"^customers/(?P<customer_id>.+?)/campaigns/(?P<campaign_id>.+?)$",
            path,
        )
        return m.groupdict() if m else {}

    @staticmethod
    def campaign_draft_path(
        customer_id: str, base_campaign_id: str, draft_id: str,
    ) -> str:
        """Returns a fully-qualified campaign_draft string."""
        return "customers/{customer_id}/campaignDrafts/{base_campaign_id}~{draft_id}".format(
            customer_id=customer_id,
            base_campaign_id=base_campaign_id,
            draft_id=draft_id,
        )

    @staticmethod
    def parse_campaign_draft_path(path: str) -> Dict[str, str]:
        """Parses a campaign_draft path into its component segments."""
        m = re.match(
            r"^customers/(?P<customer_id>.+?)/campaignDrafts/(?P<base_campaign_id>.+?)~(?P<draft_id>.+?)$",
            path,
        )
        return m.groupdict() if m else {}

    @staticmethod
    def common_billing_account_path(billing_account: str,) -> str:
        """Returns a fully-qualified billing_account string."""
        return "billingAccounts/{billing_account}".format(
            billing_account=billing_account,
        )

    @staticmethod
    def parse_common_billing_account_path(path: str) -> Dict[str, str]:
        """Parse a billing_account path into its component segments."""
        m = re.match(r"^billingAccounts/(?P<billing_account>.+?)$", path)
        return m.groupdict() if m else {}

    @staticmethod
    def common_folder_path(folder: str,) -> str:
        """Returns a fully-qualified folder string."""
        return "folders/{folder}".format(folder=folder,)

    @staticmethod
    def parse_common_folder_path(path: str) -> Dict[str, str]:
        """Parse a folder path into its component segments."""
        m = re.match(r"^folders/(?P<folder>.+?)$", path)
        return m.groupdict() if m else {}

    @staticmethod
    def common_organization_path(organization: str,) -> str:
        """Returns a fully-qualified organization string."""
        return "organizations/{organization}".format(organization=organization,)

    @staticmethod
    def parse_common_organization_path(path: str) -> Dict[str, str]:
        """Parse a organization path into its component segments."""
        m = re.match(r"^organizations/(?P<organization>.+?)$", path)
        return m.groupdict() if m else {}

    @staticmethod
    def common_project_path(project: str,) -> str:
        """Returns a fully-qualified project string."""
        return "projects/{project}".format(project=project,)

    @staticmethod
    def parse_common_project_path(path: str) -> Dict[str, str]:
        """Parse a project path into its component segments."""
        m = re.match(r"^projects/(?P<project>.+?)$", path)
        return m.groupdict() if m else {}

    @staticmethod
    def common_location_path(project: str, location: str,) -> str:
        """Returns a fully-qualified location string."""
        return "projects/{project}/locations/{location}".format(
            project=project, location=location,
        )

    @staticmethod
    def parse_common_location_path(path: str) -> Dict[str, str]:
        """Parse a location path into its component segments."""
        m = re.match(
            r"^projects/(?P<project>.+?)/locations/(?P<location>.+?)$", path
        )
        return m.groupdict() if m else {}

    def __init__(
        self,
        *,
        credentials: Optional[ga_credentials.Credentials] = None,
        transport: Union[str, CampaignDraftServiceTransport, None] = None,
        client_options: Optional[client_options_lib.ClientOptions] = None,
        client_info: gapic_v1.client_info.ClientInfo = DEFAULT_CLIENT_INFO,
    ) -> None:
        """Instantiates the campaign draft service client.

        Args:
            credentials (Optional[google.auth.credentials.Credentials]): The
                authorization credentials to attach to requests. These
                credentials identify the application to the service; if none
                are specified, the client will attempt to ascertain the
                credentials from the environment.
            transport (Union[str, CampaignDraftServiceTransport]): The
                transport to use. If set to None, a transport is chosen
                automatically.
            client_options (google.api_core.client_options.ClientOptions): Custom options for the
                client. It won't take effect if a ``transport`` instance is provided.
                (1) The ``api_endpoint`` property can be used to override the
                default endpoint provided by the client. GOOGLE_API_USE_MTLS_ENDPOINT
                environment variable can also be used to override the endpoint:
                "always" (always use the default mTLS endpoint), "never" (always
                use the default regular endpoint) and "auto" (auto switch to the
                default mTLS endpoint if client certificate is present, this is
                the default value). However, the ``api_endpoint`` property takes
                precedence if provided.
                (2) If GOOGLE_API_USE_CLIENT_CERTIFICATE environment variable
                is "true", then the ``client_cert_source`` property can be used
                to provide client certificate for mutual TLS transport. If
                not provided, the default SSL client certificate will be used if
                present. If GOOGLE_API_USE_CLIENT_CERTIFICATE is "false" or not
                set, no client certificate will be used.
            client_info (google.api_core.gapic_v1.client_info.ClientInfo):
                The client info used to send a user-agent string along with
                API requests. If ``None``, then default info will be used.
                Generally, you only need to set this if you're developing
                your own client library.

        Raises:
            google.auth.exceptions.MutualTLSChannelError: If mutual TLS transport
                creation failed for any reason.
        """
        if isinstance(client_options, dict):
            client_options = client_options_lib.from_dict(client_options)
        if client_options is None:
            client_options = client_options_lib.ClientOptions()

        # Create SSL credentials for mutual TLS if needed.
        if os.getenv("GOOGLE_API_USE_CLIENT_CERTIFICATE", "false") not in (
            "true",
            "false",
        ):
            raise ValueError(
                "Environment variable `GOOGLE_API_USE_CLIENT_CERTIFICATE` must be either `true` or `false`"
            )
        use_client_cert = (
            os.getenv("GOOGLE_API_USE_CLIENT_CERTIFICATE", "false") == "true"
        )

        client_cert_source_func = None
        is_mtls = False
        if use_client_cert:
            if client_options.client_cert_source:
                is_mtls = True
                client_cert_source_func = client_options.client_cert_source
            else:
                is_mtls = mtls.has_default_client_cert_source()
                if is_mtls:
                    client_cert_source_func = mtls.default_client_cert_source()
                else:
                    client_cert_source_func = None

        # Figure out which api endpoint to use.
        if client_options.api_endpoint is not None:
            api_endpoint = client_options.api_endpoint
        else:
            use_mtls_env = os.getenv("GOOGLE_API_USE_MTLS_ENDPOINT", "auto")
            if use_mtls_env == "never":
                api_endpoint = self.DEFAULT_ENDPOINT
            elif use_mtls_env == "always":
                api_endpoint = self.DEFAULT_MTLS_ENDPOINT
            elif use_mtls_env == "auto":
                api_endpoint = (
                    self.DEFAULT_MTLS_ENDPOINT
                    if is_mtls
                    else self.DEFAULT_ENDPOINT
                )
            else:
                raise MutualTLSChannelError(
                    "Unsupported GOOGLE_API_USE_MTLS_ENDPOINT value. Accepted "
                    "values: never, auto, always"
                )

        # Save or instantiate the transport.
        # Ordinarily, we provide the transport, but allowing a custom transport
        # instance provides an extensibility point for unusual situations.
        if isinstance(transport, CampaignDraftServiceTransport):
            # transport is a CampaignDraftServiceTransport instance.
            if credentials or client_options.credentials_file:
                raise ValueError(
                    "When providing a transport instance, "
                    "provide its credentials directly."
                )
            if client_options.scopes:
                raise ValueError(
                    "When providing a transport instance, provide its scopes "
                    "directly."
                )
            self._transport = transport
        else:
            Transport = type(self).get_transport_class(transport)
            self._transport = Transport(
                credentials=credentials,
                credentials_file=client_options.credentials_file,
                host=api_endpoint,
                scopes=client_options.scopes,
                client_cert_source_for_mtls=client_cert_source_func,
                quota_project_id=client_options.quota_project_id,
                client_info=client_info,
                always_use_jwt_access=True,
            )

    def mutate_campaign_drafts(
        self,
        request: Union[
            campaign_draft_service.MutateCampaignDraftsRequest, dict
        ] = None,
        *,
        customer_id: str = None,
        operations: Sequence[
            campaign_draft_service.CampaignDraftOperation
        ] = None,
        retry: OptionalRetry = gapic_v1.method.DEFAULT,
        timeout: float = None,
        metadata: Sequence[Tuple[str, str]] = (),
    ) -> campaign_draft_service.MutateCampaignDraftsResponse:
        r"""Creates, updates, or removes campaign drafts. Operation statuses
        are returned.

        List of thrown errors: `AuthenticationError <>`__
        `AuthorizationError <>`__ `CampaignDraftError <>`__
        `DatabaseError <>`__ `FieldError <>`__ `HeaderError <>`__
        `InternalError <>`__ `MutateError <>`__ `QuotaError <>`__
        `RequestError <>`__

        Args:
            request (Union[google.ads.googleads.v11.services.types.MutateCampaignDraftsRequest, dict]):
                The request object. Request message for
                [CampaignDraftService.MutateCampaignDrafts][google.ads.googleads.v11.services.CampaignDraftService.MutateCampaignDrafts].
            customer_id (str):
                Required. The ID of the customer
                whose campaign drafts are being
                modified.

                This corresponds to the ``customer_id`` field
                on the ``request`` instance; if ``request`` is provided, this
                should not be set.
            operations (Sequence[google.ads.googleads.v11.services.types.CampaignDraftOperation]):
                Required. The list of operations to
                perform on individual campaign drafts.

                This corresponds to the ``operations`` field
                on the ``request`` instance; if ``request`` is provided, this
                should not be set.
            retry (google.api_core.retry.Retry): Designation of what errors, if any,
                should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, str]]): Strings which should be
                sent along with the request as metadata.

        Returns:
            google.ads.googleads.v11.services.types.MutateCampaignDraftsResponse:
                Response message for campaign draft
                mutate.

        """
        # Create or coerce a protobuf request object.
        # Quick check: If we got a request object, we should *not* have
        # gotten any keyword arguments that map to the request.
        has_flattened_params = any([customer_id, operations])
        if request is not None and has_flattened_params:
            raise ValueError(
                "If the `request` argument is set, then none of "
                "the individual field arguments should be set."
            )

        # Minor optimization to avoid making a copy if the user passes
        # in a campaign_draft_service.MutateCampaignDraftsRequest.
        # There's no risk of modifying the input as we've already verified
        # there are no flattened fields.
        if not isinstance(
            request, campaign_draft_service.MutateCampaignDraftsRequest
        ):
            request = campaign_draft_service.MutateCampaignDraftsRequest(
                request
            )
            # If we have keyword arguments corresponding to fields on the
            # request, apply these.
            if customer_id is not None:
                request.customer_id = customer_id
            if operations is not None:
                request.operations = operations

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = self._transport._wrapped_methods[
            self._transport.mutate_campaign_drafts
        ]

        # Certain fields should be provided within the metadata header;
        # add these here.
        metadata = tuple(metadata) + (
            gapic_v1.routing_header.to_grpc_metadata(
                (("customer_id", request.customer_id),)
            ),
        )

        # Send the request.
        response = rpc(
            request, retry=retry, timeout=timeout, metadata=metadata,
        )

        # Done; return the response.
        return response

    def promote_campaign_draft(
        self,
        request: Union[
            campaign_draft_service.PromoteCampaignDraftRequest, dict
        ] = None,
        *,
        campaign_draft: str = None,
        retry: OptionalRetry = gapic_v1.method.DEFAULT,
        timeout: float = None,
        metadata: Sequence[Tuple[str, str]] = (),
    ) -> operation.Operation:
        r"""Promotes the changes in a draft back to the base campaign.

        This method returns a Long Running Operation (LRO) indicating if
        the Promote is done. Use [Operations.GetOperation] to poll the
        LRO until it is done. Only a done status is returned in the
        response. See the status in the Campaign Draft resource to
        determine if the promotion was successful. If the LRO failed,
        use
        [CampaignDraftService.ListCampaignDraftAsyncErrors][google.ads.googleads.v11.services.CampaignDraftService.ListCampaignDraftAsyncErrors]
        to view the list of error reasons.

        List of thrown errors: `AuthenticationError <>`__
        `AuthorizationError <>`__ `CampaignDraftError <>`__
        `HeaderError <>`__ `InternalError <>`__ `QuotaError <>`__
        `RequestError <>`__

        Args:
            request (Union[google.ads.googleads.v11.services.types.PromoteCampaignDraftRequest, dict]):
                The request object. Request message for
                [CampaignDraftService.PromoteCampaignDraft][google.ads.googleads.v11.services.CampaignDraftService.PromoteCampaignDraft].
            campaign_draft (str):
                Required. The resource name of the
                campaign draft to promote.

                This corresponds to the ``campaign_draft`` field
                on the ``request`` instance; if ``request`` is provided, this
                should not be set.
            retry (google.api_core.retry.Retry): Designation of what errors, if any,
                should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, str]]): Strings which should be
                sent along with the request as metadata.

        Returns:
            google.api_core.operation.Operation:
                An object representing a long-running operation.

                The result type for the operation will be :class:`google.protobuf.empty_pb2.Empty` A generic empty message that you can re-use to avoid defining duplicated
                   empty messages in your APIs. A typical example is to
                   use it as the request or the response type of an API
                   method. For instance:

                      service Foo {
                         rpc Bar(google.protobuf.Empty) returns
                         (google.protobuf.Empty);

                      }

                   The JSON representation for Empty is empty JSON
                   object {}.

        """
        # Create or coerce a protobuf request object.
        # Quick check: If we got a request object, we should *not* have
        # gotten any keyword arguments that map to the request.
        has_flattened_params = any([campaign_draft])
        if request is not None and has_flattened_params:
            raise ValueError(
                "If the `request` argument is set, then none of "
                "the individual field arguments should be set."
            )

        # Minor optimization to avoid making a copy if the user passes
        # in a campaign_draft_service.PromoteCampaignDraftRequest.
        # There's no risk of modifying the input as we've already verified
        # there are no flattened fields.
        if not isinstance(
            request, campaign_draft_service.PromoteCampaignDraftRequest
        ):
            request = campaign_draft_service.PromoteCampaignDraftRequest(
                request
            )
            # If we have keyword arguments corresponding to fields on the
            # request, apply these.
            if campaign_draft is not None:
                request.campaign_draft = campaign_draft

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = self._transport._wrapped_methods[
            self._transport.promote_campaign_draft
        ]

        # Certain fields should be provided within the metadata header;
        # add these here.
        metadata = tuple(metadata) + (
            gapic_v1.routing_header.to_grpc_metadata(
                (("campaign_draft", request.campaign_draft),)
            ),
        )

        # Send the request.
        response = rpc(
            request, retry=retry, timeout=timeout, metadata=metadata,
        )

        # Wrap the response in an operation future.
        response = operation.from_gapic(
            response,
            self._transport.operations_client,
            empty_pb2.Empty,
            metadata_type=empty_pb2.Empty,
        )

        # Done; return the response.
        return response

    def list_campaign_draft_async_errors(
        self,
        request: Union[
            campaign_draft_service.ListCampaignDraftAsyncErrorsRequest, dict
        ] = None,
        *,
        resource_name: str = None,
        retry: OptionalRetry = gapic_v1.method.DEFAULT,
        timeout: float = None,
        metadata: Sequence[Tuple[str, str]] = (),
    ) -> pagers.ListCampaignDraftAsyncErrorsPager:
        r"""Returns all errors that occurred during CampaignDraft promote.
        Throws an error if called before campaign draft is promoted.
        Supports standard list paging.

        List of thrown errors: `AuthenticationError <>`__
        `AuthorizationError <>`__ `HeaderError <>`__
        `InternalError <>`__ `QuotaError <>`__ `RequestError <>`__

        Args:
            request (Union[google.ads.googleads.v11.services.types.ListCampaignDraftAsyncErrorsRequest, dict]):
                The request object. Request message for
                [CampaignDraftService.ListCampaignDraftAsyncErrors][google.ads.googleads.v11.services.CampaignDraftService.ListCampaignDraftAsyncErrors].
            resource_name (str):
                Required. The name of the campaign
                draft from which to retrieve the async
                errors.

                This corresponds to the ``resource_name`` field
                on the ``request`` instance; if ``request`` is provided, this
                should not be set.
            retry (google.api_core.retry.Retry): Designation of what errors, if any,
                should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, str]]): Strings which should be
                sent along with the request as metadata.

        Returns:
            google.ads.googleads.v11.services.services.campaign_draft_service.pagers.ListCampaignDraftAsyncErrorsPager:
                Response message for
                [CampaignDraftService.ListCampaignDraftAsyncErrors][google.ads.googleads.v11.services.CampaignDraftService.ListCampaignDraftAsyncErrors].

                Iterating over this object will yield results and
                resolve additional pages automatically.

        """
        # Create or coerce a protobuf request object.
        # Quick check: If we got a request object, we should *not* have
        # gotten any keyword arguments that map to the request.
        has_flattened_params = any([resource_name])
        if request is not None and has_flattened_params:
            raise ValueError(
                "If the `request` argument is set, then none of "
                "the individual field arguments should be set."
            )

        # Minor optimization to avoid making a copy if the user passes
        # in a campaign_draft_service.ListCampaignDraftAsyncErrorsRequest.
        # There's no risk of modifying the input as we've already verified
        # there are no flattened fields.
        if not isinstance(
            request, campaign_draft_service.ListCampaignDraftAsyncErrorsRequest
        ):
            request = campaign_draft_service.ListCampaignDraftAsyncErrorsRequest(
                request
            )
            # If we have keyword arguments corresponding to fields on the
            # request, apply these.
            if resource_name is not None:
                request.resource_name = resource_name

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = self._transport._wrapped_methods[
            self._transport.list_campaign_draft_async_errors
        ]

        # Certain fields should be provided within the metadata header;
        # add these here.
        metadata = tuple(metadata) + (
            gapic_v1.routing_header.to_grpc_metadata(
                (("resource_name", request.resource_name),)
            ),
        )

        # Send the request.
        response = rpc(
            request, retry=retry, timeout=timeout, metadata=metadata,
        )

        # This method is paged; wrap the response in a pager, which provides
        # an `__iter__` convenience method.
        response = pagers.ListCampaignDraftAsyncErrorsPager(
            method=rpc, request=request, response=response, metadata=metadata,
        )

        # Done; return the response.
        return response


try:
    DEFAULT_CLIENT_INFO = gapic_v1.client_info.ClientInfo(
        gapic_version=pkg_resources.get_distribution("google-ads",).version,
    )
except pkg_resources.DistributionNotFound:
    DEFAULT_CLIENT_INFO = gapic_v1.client_info.ClientInfo()


__all__ = ("CampaignDraftServiceClient",)



# -*- coding: utf-8 -*-
# Copyright 2022 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v10.errors",
    marshal="google.ads.googleads.v10",
    manifest={
        "UserListErrorEnum",
    },
)


class UserListErrorEnum(proto.Message):
    r"""Container for enum describing possible user list errors."""

    class UserListError(proto.Enum):
        r"""Enum describing possible user list errors."""
        UNSPECIFIED = 0
        UNKNOWN = 1
        EXTERNAL_REMARKETING_USER_LIST_MUTATE_NOT_SUPPORTED = 2
        CONCRETE_TYPE_REQUIRED = 3
        CONVERSION_TYPE_ID_REQUIRED = 4
        DUPLICATE_CONVERSION_TYPES = 5
        INVALID_CONVERSION_TYPE = 6
        INVALID_DESCRIPTION = 7
        INVALID_NAME = 8
        INVALID_TYPE = 9
        CAN_NOT_ADD_LOGICAL_LIST_AS_LOGICAL_LIST_OPERAND = 10
        INVALID_USER_LIST_LOGICAL_RULE_OPERAND = 11
        NAME_ALREADY_USED = 12
        NEW_CONVERSION_TYPE_NAME_REQUIRED = 13
        CONVERSION_TYPE_NAME_ALREADY_USED = 14
        OWNERSHIP_REQUIRED_FOR_SET = 15
        USER_LIST_MUTATE_NOT_SUPPORTED = 16
        INVALID_RULE = 17
        INVALID_DATE_RANGE = 27
        CAN_NOT_MUTATE_SENSITIVE_USERLIST = 28
        MAX_NUM_RULEBASED_USERLISTS = 29
        CANNOT_MODIFY_BILLABLE_RECORD_COUNT = 30
        APP_ID_NOT_SET = 31
        USERLIST_NAME_IS_RESERVED_FOR_SYSTEM_LIST = 32
        ADVERTISER_NOT_ON_ALLOWLIST_FOR_USING_UPLOADED_DATA = 37
        RULE_TYPE_IS_NOT_SUPPORTED = 34
        CAN_NOT_ADD_A_SIMILAR_USERLIST_AS_LOGICAL_LIST_OPERAND = 35
        CAN_NOT_MIX_CRM_BASED_IN_LOGICAL_LIST_WITH_OTHER_LISTS = 36


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2022 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v10.errors",
    marshal="google.ads.googleads.v10",
    manifest={
        "ConversionValueRuleErrorEnum",
    },
)


class ConversionValueRuleErrorEnum(proto.Message):
    r"""Container for enum describing possible conversion value rule
    errors.

    """

    class ConversionValueRuleError(proto.Enum):
        r"""Enum describing possible conversion value rule errors."""
        UNSPECIFIED = 0
        UNKNOWN = 1
        INVALID_GEO_TARGET_CONSTANT = 2
        CONFLICTING_INCLUDED_AND_EXCLUDED_GEO_TARGET = 3
        CONFLICTING_CONDITIONS = 4
        CANNOT_REMOVE_IF_INCLUDED_IN_VALUE_RULE_SET = 5
        CONDITION_NOT_ALLOWED = 6
        FIELD_MUST_BE_UNSET = 7
        CANNOT_PAUSE_UNLESS_VALUE_RULE_SET_IS_PAUSED = 8
        UNTARGETABLE_GEO_TARGET = 9
        INVALID_AUDIENCE_USER_LIST = 10
        INACCESSIBLE_USER_LIST = 11
        INVALID_AUDIENCE_USER_INTEREST = 12
        CANNOT_ADD_RULE_WITH_STATUS_REMOVED = 13


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2022 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v10.enums",
    marshal="google.ads.googleads.v10",
    manifest={
        "PlaceholderTypeEnum",
    },
)


class PlaceholderTypeEnum(proto.Message):
    r"""Container for enum describing possible placeholder types for
    a feed mapping.

    """

    class PlaceholderType(proto.Enum):
        r"""Possible placeholder types for a feed mapping."""
        UNSPECIFIED = 0
        UNKNOWN = 1
        SITELINK = 2
        CALL = 3
        APP = 4
        LOCATION = 5
        AFFILIATE_LOCATION = 6
        CALLOUT = 7
        STRUCTURED_SNIPPET = 8
        MESSAGE = 9
        PRICE = 10
        PROMOTION = 11
        AD_CUSTOMIZER = 12
        DYNAMIC_EDUCATION = 13
        DYNAMIC_FLIGHT = 14
        DYNAMIC_CUSTOM = 15
        DYNAMIC_HOTEL = 16
        DYNAMIC_REAL_ESTATE = 17
        DYNAMIC_TRAVEL = 18
        DYNAMIC_LOCAL = 19
        DYNAMIC_JOB = 20
        IMAGE = 21


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2022 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
import proto  # type: ignore

from google.ads.googleads.v11.enums.types import interaction_event_type
from google.ads.googleads.v11.enums.types import quality_score_bucket


__protobuf__ = proto.module(
    package="google.ads.googleads.v11.common",
    marshal="google.ads.googleads.v11",
    manifest={
        "Metrics",
    },
)


class Metrics(proto.Message):
    r"""Metrics data.

    Attributes:
        absolute_top_impression_percentage (float):
            The percent of your ad impressions that are
            shown as the very first ad above the organic
            search results.

            This field is a member of `oneof`_ ``_absolute_top_impression_percentage``.
        active_view_cpm (float):
            Average cost of viewable impressions
            (``active_view_impressions``).

            This field is a member of `oneof`_ ``_active_view_cpm``.
        active_view_ctr (float):
            Active view measurable clicks divided by
            active view viewable impressions. This metric is
            reported only for display network.

            This field is a member of `oneof`_ ``_active_view_ctr``.
        active_view_impressions (int):
            A measurement of how often your ad has become
            viewable on a Display Network site.

            This field is a member of `oneof`_ ``_active_view_impressions``.
        active_view_measurability (float):
            The ratio of impressions that could be
            measured by Active View over the number of
            served impressions.

            This field is a member of `oneof`_ ``_active_view_measurability``.
        active_view_measurable_cost_micros (int):
            The cost of the impressions you received that
            were measurable by Active View.

            This field is a member of `oneof`_ ``_active_view_measurable_cost_micros``.
        active_view_measurable_impressions (int):
            The number of times your ads are appearing on
            placements in positions where they can be seen.

            This field is a member of `oneof`_ ``_active_view_measurable_impressions``.
        active_view_viewability (float):
            The percentage of time when your ad appeared
            on an Active View enabled site (measurable
            impressions) and was viewable (viewable
            impressions).

            This field is a member of `oneof`_ ``_active_view_viewability``.
        all_conversions_from_interactions_rate (float):
            All conversions from interactions (as oppose
            to view through conversions) divided by the
            number of ad interactions.

            This field is a member of `oneof`_ ``_all_conversions_from_interactions_rate``.
        all_conversions_value (float):
            The value of all conversions.

            This field is a member of `oneof`_ ``_all_conversions_value``.
        all_conversions_value_by_conversion_date (float):
            The value of all conversions. When this column is selected
            with date, the values in date column means the conversion
            date. Details for the by_conversion_date columns are
            available at
            https://support.google.com/google-ads/answer/9549009.
        all_conversions (float):
            The total number of conversions. This includes all
            conversions regardless of the value of
            include_in_conversions_metric.

            This field is a member of `oneof`_ ``_all_conversions``.
        all_conversions_by_conversion_date (float):
            The total number of conversions. This includes all
            conversions regardless of the value of
            include_in_conversions_metric. When this column is selected
            with date, the values in date column means the conversion
            date. Details for the by_conversion_date columns are
            available at
            https://support.google.com/google-ads/answer/9549009.
        all_conversions_value_per_cost (float):
            The value of all conversions divided by the
            total cost of ad interactions (such as clicks
            for text ads or views for video ads).

            This field is a member of `oneof`_ ``_all_conversions_value_per_cost``.
        all_conversions_from_click_to_call (float):
            The number of times people clicked the "Call"
            button to call a store during or after clicking
            an ad. This number doesn't include whether or
            not calls were connected, or the duration of any
            calls. This metric applies to feed items only.

            This field is a member of `oneof`_ ``_all_conversions_from_click_to_call``.
        all_conversions_from_directions (float):
            The number of times people clicked a "Get
            directions" button to navigate to a store after
            clicking an ad. This metric applies to feed
            items only.

            This field is a member of `oneof`_ ``_all_conversions_from_directions``.
        all_conversions_from_interactions_value_per_interaction (float):
            The value of all conversions from
            interactions divided by the total number of
            interactions.

            This field is a member of `oneof`_ ``_all_conversions_from_interactions_value_per_interaction``.
        all_conversions_from_menu (float):
            The number of times people clicked a link to
            view a store's menu after clicking an ad.
            This metric applies to feed items only.

            This field is a member of `oneof`_ ``_all_conversions_from_menu``.
        all_conversions_from_order (float):
            The number of times people placed an order at
            a store after clicking an ad. This metric
            applies to feed items only.

            This field is a member of `oneof`_ ``_all_conversions_from_order``.
        all_conversions_from_other_engagement (float):
            The number of other conversions (for example,
            posting a review or saving a location for a
            store) that occurred after people clicked an ad.
            This metric applies to feed items only.

            This field is a member of `oneof`_ ``_all_conversions_from_other_engagement``.
        all_conversions_from_store_visit (float):
            Estimated number of times people visited a
            store after clicking an ad. This metric applies
            to feed items only.

            This field is a member of `oneof`_ ``_all_conversions_from_store_visit``.
        all_conversions_from_store_website (float):
            The number of times that people were taken to
            a store's URL after clicking an ad.
            This metric applies to feed items only.

            This field is a member of `oneof`_ ``_all_conversions_from_store_website``.
        auction_insight_search_absolute_top_impression_percentage (float):
            This metric is part of the Auction Insights
            report, and tells how often the ads of another
            participant showed as the very first ad above
            the organic search results.
            This percentage is computed only over the
            auctions that you appeared in the page.
            This metric is not publicly available.

            This field is a member of `oneof`_ ``_auction_insight_search_absolute_top_impression_percentage``.
        auction_insight_search_impression_share (float):
            This metric is part of the Auction Insights
            report, and tells the percentage of impressions
            that another participant obtained, over the
            total number of impressions that your ads were
            eligible for. Any value below 0.1 is reported as
            0.0999.
            This metric is not publicly available.

            This field is a member of `oneof`_ ``_auction_insight_search_impression_share``.
        auction_insight_search_outranking_share (float):
            This metric is part of the Auction Insights
            report, and tells the percentage of impressions
            that your ads outranked (showed above) another
            participant in the auction, compared to the
            total number of impressions that your ads were
            eligible for.
            Any value below 0.1 is reported as 0.0999.
            This metric is not publicly available.

            This field is a member of `oneof`_ ``_auction_insight_search_outranking_share``.
        auction_insight_search_overlap_rate (float):
            This metric is part of the Auction Insights
            report, and tells how often another
            participant's ad received an impression when
            your ad also received an impression.
            This metric is not publicly available.

            This field is a member of `oneof`_ ``_auction_insight_search_overlap_rate``.
        auction_insight_search_position_above_rate (float):
            This metric is part of the Auction Insights
            report, and tells how often another
            participant's ad was shown in a higher position
            than yours, when both of your ads were shown at
            the same page. This metric is not publicly
            available.

            This field is a member of `oneof`_ ``_auction_insight_search_position_above_rate``.
        auction_insight_search_top_impression_percentage (float):
            This metric is part of the Auction Insights
            report, and tells how often the ads of another
            participant showed above the organic search
            results. This percentage is computed only over
            the auctions that you appeared in the page.
            This metric is not publicly available.

            This field is a member of `oneof`_ ``_auction_insight_search_top_impression_percentage``.
        average_cost (float):
            The average amount you pay per interaction.
            This amount is the total cost of your ads
            divided by the total number of interactions.

            This field is a member of `oneof`_ ``_average_cost``.
        average_cpc (float):
            The total cost of all clicks divided by the
            total number of clicks received.

            This field is a member of `oneof`_ ``_average_cpc``.
        average_cpe (float):
            The average amount that you've been charged
            for an ad engagement. This amount is the total
            cost of all ad engagements divided by the total
            number of ad engagements.

            This field is a member of `oneof`_ ``_average_cpe``.
        average_cpm (float):
            Average cost-per-thousand impressions (CPM).

            This field is a member of `oneof`_ ``_average_cpm``.
        average_cpv (float):
            The average amount you pay each time someone
            views your ad. The average CPV is defined by the
            total cost of all ad views divided by the number
            of views.

            This field is a member of `oneof`_ ``_average_cpv``.
        average_page_views (float):
            Average number of pages viewed per session.

            This field is a member of `oneof`_ ``_average_page_views``.
        average_time_on_site (float):
            Total duration of all sessions (in seconds) /
            number of sessions. Imported from Google
            Analytics.

            This field is a member of `oneof`_ ``_average_time_on_site``.
        benchmark_average_max_cpc (float):
            An indication of how other advertisers are
            bidding on similar products.

            This field is a member of `oneof`_ ``_benchmark_average_max_cpc``.
        biddable_app_install_conversions (float):
            Number of app installs.

            This field is a member of `oneof`_ ``_biddable_app_install_conversions``.
        biddable_app_post_install_conversions (float):
            Number of in-app actions.

            This field is a member of `oneof`_ ``_biddable_app_post_install_conversions``.
        benchmark_ctr (float):
            An indication on how other advertisers'
            Shopping ads for similar products are performing
            based on how often people who see their ad click
            on it.

            This field is a member of `oneof`_ ``_benchmark_ctr``.
        bounce_rate (float):
            Percentage of clicks where the user only
            visited a single page on your site. Imported
            from Google Analytics.

            This field is a member of `oneof`_ ``_bounce_rate``.
        clicks (int):
            The number of clicks.

            This field is a member of `oneof`_ ``_clicks``.
        combined_clicks (int):
            The number of times your ad or your site's
            listing in the unpaid results was clicked. See
            the help page at
            https://support.google.com/google-ads/answer/3097241
            for details.

            This field is a member of `oneof`_ ``_combined_clicks``.
        combined_clicks_per_query (float):
            The number of times your ad or your site's listing in the
            unpaid results was clicked (combined_clicks) divided by
            combined_queries. See the help page at
            https://support.google.com/google-ads/answer/3097241 for
            details.

            This field is a member of `oneof`_ ``_combined_clicks_per_query``.
        combined_queries (int):
            The number of searches that returned pages
            from your site in the unpaid results or showed
            one of your text ads. See the help page at
            https://support.google.com/google-ads/answer/3097241
            for details.

            This field is a member of `oneof`_ ``_combined_queries``.
        content_budget_lost_impression_share (float):
            The estimated percent of times that your ad
            was eligible to show on the Display Network but
            didn't because your budget was too low. Note:
            Content budget lost impression share is reported
            in the range of 0 to 0.9. Any value above 0.9 is
            reported as 0.9001.

            This field is a member of `oneof`_ ``_content_budget_lost_impression_share``.
        content_impression_share (float):
            The impressions you've received on the
            Display Network divided by the estimated number
            of impressions you were eligible to receive.
            Note: Content impression share is reported in
            the range of 0.1 to 1. Any value below 0.1 is
            reported as 0.0999.

            This field is a member of `oneof`_ ``_content_impression_share``.
        conversion_last_received_request_date_time (str):
            The last date/time a conversion tag for this
            conversion action successfully fired and was
            seen by Google Ads. This firing event may not
            have been the result of an attributable
            conversion (e.g. because the tag was fired from
            a browser that did not previously click an ad
            from an appropriate advertiser). The date/time
            is in the customer's time zone.

            This field is a member of `oneof`_ ``_conversion_last_received_request_date_time``.
        conversion_last_conversion_date (str):
            The date of the most recent conversion for
            this conversion action. The date is in the
            customer's time zone.

            This field is a member of `oneof`_ ``_conversion_last_conversion_date``.
        content_rank_lost_impression_share (float):
            The estimated percentage of impressions on
            the Display Network that your ads didn't receive
            due to poor Ad Rank. Note: Content rank lost
            impression share is reported in the range of 0
            to 0.9. Any value above 0.9 is reported as
            0.9001.

            This field is a member of `oneof`_ ``_content_rank_lost_impression_share``.
        conversions_from_interactions_rate (float):
            Conversions from interactions divided by the number of ad
            interactions (such as clicks for text ads or views for video
            ads). This only includes conversion actions which
            include_in_conversions_metric attribute is set to true. If
            you use conversion-based bidding, your bid strategies will
            optimize for these conversions.

            This field is a member of `oneof`_ ``_conversions_from_interactions_rate``.
        conversions_value (float):
            The value of conversions. This only includes conversion
            actions which include_in_conversions_metric attribute is set
            to true. If you use conversion-based bidding, your bid
            strategies will optimize for these conversions.

            This field is a member of `oneof`_ ``_conversions_value``.
        conversions_value_by_conversion_date (float):
            The value of conversions. This only includes conversion
            actions which include_in_conversions_metric attribute is set
            to true. If you use conversion-based bidding, your bid
            strategies will optimize for these conversions. When this
            column is selected with date, the values in date column
            means the conversion date. Details for the
            by_conversion_date columns are available at
            https://support.google.com/google-ads/answer/9549009.
        conversions_value_per_cost (float):
            The value of conversions divided by the cost of ad
            interactions. This only includes conversion actions which
            include_in_conversions_metric attribute is set to true. If
            you use conversion-based bidding, your bid strategies will
            optimize for these conversions.

            This field is a member of `oneof`_ ``_conversions_value_per_cost``.
        conversions_from_interactions_value_per_interaction (float):
            The value of conversions from interactions divided by the
            number of ad interactions. This only includes conversion
            actions which include_in_conversions_metric attribute is set
            to true. If you use conversion-based bidding, your bid
            strategies will optimize for these conversions.

            This field is a member of `oneof`_ ``_conversions_from_interactions_value_per_interaction``.
        conversions (float):
            The number of conversions. This only includes conversion
            actions which include_in_conversions_metric attribute is set
            to true. If you use conversion-based bidding, your bid
            strategies will optimize for these conversions.

            This field is a member of `oneof`_ ``_conversions``.
        conversions_by_conversion_date (float):
            The number of conversions. This only includes conversion
            actions which include_in_conversions_metric attribute is set
            to true. If you use conversion-based bidding, your bid
            strategies will optimize for these conversions. When this
            column is selected with date, the values in date column
            means the conversion date. Details for the
            by_conversion_date columns are available at
            https://support.google.com/google-ads/answer/9549009.
        cost_micros (int):
            The sum of your cost-per-click (CPC) and
            cost-per-thousand impressions (CPM) costs during
            this period.

            This field is a member of `oneof`_ ``_cost_micros``.
        cost_per_all_conversions (float):
            The cost of ad interactions divided by all
            conversions.

            This field is a member of `oneof`_ ``_cost_per_all_conversions``.
        cost_per_conversion (float):
            The cost of ad interactions divided by conversions. This
            only includes conversion actions which
            include_in_conversions_metric attribute is set to true. If
            you use conversion-based bidding, your bid strategies will
            optimize for these conversions.

            This field is a member of `oneof`_ ``_cost_per_conversion``.
        cost_per_current_model_attributed_conversion (float):
            The cost of ad interactions divided by current model
            attributed conversions. This only includes conversion
            actions which include_in_conversions_metric attribute is set
            to true. If you use conversion-based bidding, your bid
            strategies will optimize for these conversions.

            This field is a member of `oneof`_ ``_cost_per_current_model_attributed_conversion``.
        cross_device_conversions (float):
            Conversions from when a customer clicks on a Google Ads ad
            on one device, then converts on a different device or
            browser. Cross-device conversions are already included in
            all_conversions.

            This field is a member of `oneof`_ ``_cross_device_conversions``.
        ctr (float):
            The number of clicks your ad receives
            (Clicks) divided by the number of times your ad
            is shown (Impressions).

            This field is a member of `oneof`_ ``_ctr``.
        current_model_attributed_conversions (float):
            Shows how your historic conversions data would look under
            the attribution model you've currently selected. This only
            includes conversion actions which
            include_in_conversions_metric attribute is set to true. If
            you use conversion-based bidding, your bid strategies will
            optimize for these conversions.

            This field is a member of `oneof`_ ``_current_model_attributed_conversions``.
        current_model_attributed_conversions_from_interactions_rate (float):
            Current model attributed conversions from interactions
            divided by the number of ad interactions (such as clicks for
            text ads or views for video ads). This only includes
            conversion actions which include_in_conversions_metric
            attribute is set to true. If you use conversion-based
            bidding, your bid strategies will optimize for these
            conversions.

            This field is a member of `oneof`_ ``_current_model_attributed_conversions_from_interactions_rate``.
        current_model_attributed_conversions_from_interactions_value_per_interaction (float):
            The value of current model attributed conversions from
            interactions divided by the number of ad interactions. This
            only includes conversion actions which
            include_in_conversions_metric attribute is set to true. If
            you use conversion-based bidding, your bid strategies will
            optimize for these conversions.

            This field is a member of `oneof`_ ``_current_model_attributed_conversions_from_interactions_value_per_interaction``.
        current_model_attributed_conversions_value (float):
            The value of current model attributed conversions. This only
            includes conversion actions which
            include_in_conversions_metric attribute is set to true. If
            you use conversion-based bidding, your bid strategies will
            optimize for these conversions.

            This field is a member of `oneof`_ ``_current_model_attributed_conversions_value``.
        current_model_attributed_conversions_value_per_cost (float):
            The value of current model attributed conversions divided by
            the cost of ad interactions. This only includes conversion
            actions which include_in_conversions_metric attribute is set
            to true. If you use conversion-based bidding, your bid
            strategies will optimize for these conversions.

            This field is a member of `oneof`_ ``_current_model_attributed_conversions_value_per_cost``.
        engagement_rate (float):
            How often people engage with your ad after
            it's shown to them. This is the number of ad
            expansions divided by the number of times your
            ad is shown.

            This field is a member of `oneof`_ ``_engagement_rate``.
        engagements (int):
            The number of engagements.
            An engagement occurs when a viewer expands your
            Lightbox ad. Also, in the future, other ad types
            may support engagement metrics.

            This field is a member of `oneof`_ ``_engagements``.
        hotel_average_lead_value_micros (float):
            Average lead value based on clicks.

            This field is a member of `oneof`_ ``_hotel_average_lead_value_micros``.
        hotel_commission_rate_micros (int):
            Commission bid rate in micros. A 20%
            commission is represented as 200,000.

            This field is a member of `oneof`_ ``_hotel_commission_rate_micros``.
        hotel_expected_commission_cost (float):
            Expected commission cost. The result of multiplying the
            commission value times the hotel_commission_rate in
            advertiser currency.

            This field is a member of `oneof`_ ``_hotel_expected_commission_cost``.
        hotel_price_difference_percentage (float):
            The average price difference between the
            price offered by reporting hotel advertiser and
            the cheapest price offered by the competing
            advertiser.

            This field is a member of `oneof`_ ``_hotel_price_difference_percentage``.
        hotel_eligible_impressions (int):
            The number of impressions that hotel partners
            could have had given their feed performance.

            This field is a member of `oneof`_ ``_hotel_eligible_impressions``.
        historical_creative_quality_score (google.ads.googleads.v11.enums.types.QualityScoreBucketEnum.QualityScoreBucket):
            The creative historical quality score.
        historical_landing_page_quality_score (google.ads.googleads.v11.enums.types.QualityScoreBucketEnum.QualityScoreBucket):
            The quality of historical landing page
            experience.
        historical_quality_score (int):
            The historical quality score.

            This field is a member of `oneof`_ ``_historical_quality_score``.
        historical_search_predicted_ctr (google.ads.googleads.v11.enums.types.QualityScoreBucketEnum.QualityScoreBucket):
            The historical search predicted click through
            rate (CTR).
        gmail_forwards (int):
            The number of times the ad was forwarded to
            someone else as a message.

            This field is a member of `oneof`_ ``_gmail_forwards``.
        gmail_saves (int):
            The number of times someone has saved your
            Gmail ad to their inbox as a message.

            This field is a member of `oneof`_ ``_gmail_saves``.
        gmail_secondary_clicks (int):
            The number of clicks to the landing page on
            the expanded state of Gmail ads.

            This field is a member of `oneof`_ ``_gmail_secondary_clicks``.
        impressions_from_store_reach (int):
            The number of times a store's location-based
            ad was shown. This metric applies to feed items
            only.

            This field is a member of `oneof`_ ``_impressions_from_store_reach``.
        impressions (int):
            Count of how often your ad has appeared on a
            search results page or website on the Google
            Network.

            This field is a member of `oneof`_ ``_impressions``.
        interaction_rate (float):
            How often people interact with your ad after
            it is shown to them. This is the number of
            interactions divided by the number of times your
            ad is shown.

            This field is a member of `oneof`_ ``_interaction_rate``.
        interactions (int):
            The number of interactions.
            An interaction is the main user action
            associated with an ad format-clicks for text and
            shopping ads, views for video ads, and so on.

            This field is a member of `oneof`_ ``_interactions``.
        interaction_event_types (Sequence[google.ads.googleads.v11.enums.types.InteractionEventTypeEnum.InteractionEventType]):
            The types of payable and free interactions.
        invalid_click_rate (float):
            The percentage of clicks filtered out of your
            total number of clicks (filtered + non-filtered
            clicks) during the reporting period.

            This field is a member of `oneof`_ ``_invalid_click_rate``.
        invalid_clicks (int):
            Number of clicks Google considers
            illegitimate and doesn't charge you for.

            This field is a member of `oneof`_ ``_invalid_clicks``.
        message_chats (int):
            Number of message chats initiated for Click
            To Message impressions that were message
            tracking eligible.

            This field is a member of `oneof`_ ``_message_chats``.
        message_impressions (int):
            Number of Click To Message impressions that
            were message tracking eligible.

            This field is a member of `oneof`_ ``_message_impressions``.
        message_chat_rate (float):
            Number of message chats initiated (message_chats) divided by
            the number of message impressions (message_impressions).
            Rate at which a user initiates a message chat from an ad
            impression with a messaging option and message tracking
            enabled. Note that this rate can be more than 1.0 for a
            given message impression.

            This field is a member of `oneof`_ ``_message_chat_rate``.
        mobile_friendly_clicks_percentage (float):
            The percentage of mobile clicks that go to a
            mobile-friendly page.

            This field is a member of `oneof`_ ``_mobile_friendly_clicks_percentage``.
        optimization_score_uplift (float):
            Total optimization score uplift of all
            recommendations.

            This field is a member of `oneof`_ ``_optimization_score_uplift``.
        optimization_score_url (str):
            URL for the optimization score page in the Google Ads web
            interface. This metric can be selected from ``customer`` or
            ``campaign``, and can be segmented by
            ``segments.recommendation_type``. For example,
            ``SELECT metrics.optimization_score_url, segments.recommendation_type FROM customer``
            will return a URL for each unique (customer,
            recommendation_type) combination.

            This field is a member of `oneof`_ ``_optimization_score_url``.
        organic_clicks (int):
            The number of times someone clicked your
            site's listing in the unpaid results for a
            particular query. See the help page at
            https://support.google.com/google-ads/answer/3097241
            for details.

            This field is a member of `oneof`_ ``_organic_clicks``.
        organic_clicks_per_query (float):
            The number of times someone clicked your site's listing in
            the unpaid results (organic_clicks) divided by the total
            number of searches that returned pages from your site
            (organic_queries). See the help page at
            https://support.google.com/google-ads/answer/3097241 for
            details.

            This field is a member of `oneof`_ ``_organic_clicks_per_query``.
        organic_impressions (int):
            The number of listings for your site in the
            unpaid search results. See the help page at
            https://support.google.com/google-ads/answer/3097241
            for details.

            This field is a member of `oneof`_ ``_organic_impressions``.
        organic_impressions_per_query (float):
            The number of times a page from your site was listed in the
            unpaid search results (organic_impressions) divided by the
            number of searches returning your site's listing in the
            unpaid results (organic_queries). See the help page at
            https://support.google.com/google-ads/answer/3097241 for
            details.

            This field is a member of `oneof`_ ``_organic_impressions_per_query``.
        organic_queries (int):
            The total number of searches that returned
            your site's listing in the unpaid results. See
            the help page at
            https://support.google.com/google-ads/answer/3097241
            for details.

            This field is a member of `oneof`_ ``_organic_queries``.
        percent_new_visitors (float):
            Percentage of first-time sessions (from
            people who had never visited your site before).
            Imported from Google Analytics.

            This field is a member of `oneof`_ ``_percent_new_visitors``.
        phone_calls (int):
            Number of offline phone calls.

            This field is a member of `oneof`_ ``_phone_calls``.
        phone_impressions (int):
            Number of offline phone impressions.

            This field is a member of `oneof`_ ``_phone_impressions``.
        phone_through_rate (float):
            Number of phone calls received (phone_calls) divided by the
            number of times your phone number is shown
            (phone_impressions).

            This field is a member of `oneof`_ ``_phone_through_rate``.
        relative_ctr (float):
            Your clickthrough rate (Ctr) divided by the
            average clickthrough rate of all advertisers on
            the websites that show your ads. Measures how
            your ads perform on Display Network sites
            compared to other ads on the same sites.

            This field is a member of `oneof`_ ``_relative_ctr``.
        search_absolute_top_impression_share (float):
            The percentage of the customer's Shopping or
            Search ad impressions that are shown in the most
            prominent Shopping position. See
            https://support.google.com/google-ads/answer/7501826
            for details. Any value below 0.1 is reported as
            0.0999.

            This field is a member of `oneof`_ ``_search_absolute_top_impression_share``.
        search_budget_lost_absolute_top_impression_share (float):
            The number estimating how often your ad
            wasn't the very first ad above the organic
            search results due to a low budget. Note: Search
            budget lost absolute top impression share is
            reported in the range of 0 to 0.9. Any value
            above 0.9 is reported as 0.9001.

            This field is a member of `oneof`_ ``_search_budget_lost_absolute_top_impression_share``.
        search_budget_lost_impression_share (float):
            The estimated percent of times that your ad
            was eligible to show on the Search Network but
            didn't because your budget was too low. Note:
            Search budget lost impression share is reported
            in the range of 0 to 0.9. Any value above 0.9 is
            reported as 0.9001.

            This field is a member of `oneof`_ ``_search_budget_lost_impression_share``.
        search_budget_lost_top_impression_share (float):
            The number estimating how often your ad
            didn't show anywhere above the organic search
            results due to a low budget. Note: Search budget
            lost top impression share is reported in the
            range of 0 to 0.9. Any value above 0.9 is
            reported as 0.9001.

            This field is a member of `oneof`_ ``_search_budget_lost_top_impression_share``.
        search_click_share (float):
            The number of clicks you've received on the
            Search Network divided by the estimated number
            of clicks you were eligible to receive. Note:
            Search click share is reported in the range of
            0.1 to 1. Any value below 0.1 is reported as
            0.0999.

            This field is a member of `oneof`_ ``_search_click_share``.
        search_exact_match_impression_share (float):
            The impressions you've received divided by
            the estimated number of impressions you were
            eligible to receive on the Search Network for
            search terms that matched your keywords exactly
            (or were close variants of your keyword),
            regardless of your keyword match types. Note:
            Search exact match impression share is reported
            in the range of 0.1 to 1. Any value below 0.1 is
            reported as 0.0999.

            This field is a member of `oneof`_ ``_search_exact_match_impression_share``.
        search_impression_share (float):
            The impressions you've received on the Search
            Network divided by the estimated number of
            impressions you were eligible to receive. Note:
            Search impression share is reported in the range
            of 0.1 to 1. Any value below 0.1 is reported as
            0.0999.

            This field is a member of `oneof`_ ``_search_impression_share``.
        search_rank_lost_absolute_top_impression_share (float):
            The number estimating how often your ad
            wasn't the very first ad above the organic
            search results due to poor Ad Rank. Note: Search
            rank lost absolute top impression share is
            reported in the range of 0 to 0.9. Any value
            above 0.9 is reported as 0.9001.

            This field is a member of `oneof`_ ``_search_rank_lost_absolute_top_impression_share``.
        search_rank_lost_impression_share (float):
            The estimated percentage of impressions on
            the Search Network that your ads didn't receive
            due to poor Ad Rank. Note: Search rank lost
            impression share is reported in the range of 0
            to 0.9. Any value above 0.9 is reported as
            0.9001.

            This field is a member of `oneof`_ ``_search_rank_lost_impression_share``.
        search_rank_lost_top_impression_share (float):
            The number estimating how often your ad
            didn't show anywhere above the organic search
            results due to poor Ad Rank. Note: Search rank
            lost top impression share is reported in the
            range of 0 to 0.9. Any value above 0.9 is
            reported as 0.9001.

            This field is a member of `oneof`_ ``_search_rank_lost_top_impression_share``.
        search_top_impression_share (float):
            The impressions you've received in the top
            location (anywhere above the organic search
            results) compared to the estimated number of
            impressions you were eligible to receive in the
            top location. Note: Search top impression share
            is reported in the range of 0.1 to 1. Any value
            below 0.1 is reported as 0.0999.

            This field is a member of `oneof`_ ``_search_top_impression_share``.
        speed_score (int):
            A measure of how quickly your page loads
            after clicks on your mobile ads. The score is a
            range from 1 to 10, 10 being the fastest.

            This field is a member of `oneof`_ ``_speed_score``.
        top_impression_percentage (float):
            The percent of your ad impressions that are
            shown anywhere above the organic search results.

            This field is a member of `oneof`_ ``_top_impression_percentage``.
        valid_accelerated_mobile_pages_clicks_percentage (float):
            The percentage of ad clicks to Accelerated
            Mobile Pages (AMP) landing pages that reach a
            valid AMP page.

            This field is a member of `oneof`_ ``_valid_accelerated_mobile_pages_clicks_percentage``.
        value_per_all_conversions (float):
            The value of all conversions divided by the
            number of all conversions.

            This field is a member of `oneof`_ ``_value_per_all_conversions``.
        value_per_all_conversions_by_conversion_date (float):
            The value of all conversions divided by the number of all
            conversions. When this column is selected with date, the
            values in date column means the conversion date. Details for
            the by_conversion_date columns are available at
            https://support.google.com/google-ads/answer/9549009.

            This field is a member of `oneof`_ ``_value_per_all_conversions_by_conversion_date``.
        value_per_conversion (float):
            The value of conversions divided by the number of
            conversions. This only includes conversion actions which
            include_in_conversions_metric attribute is set to true. If
            you use conversion-based bidding, your bid strategies will
            optimize for these conversions.

            This field is a member of `oneof`_ ``_value_per_conversion``.
        value_per_conversions_by_conversion_date (float):
            The value of conversions divided by the number of
            conversions. This only includes conversion actions which
            include_in_conversions_metric attribute is set to true. If
            you use conversion-based bidding, your bid strategies will
            optimize for these conversions. When this column is selected
            with date, the values in date column means the conversion
            date. Details for the by_conversion_date columns are
            available at
            https://support.google.com/google-ads/answer/9549009.

            This field is a member of `oneof`_ ``_value_per_conversions_by_conversion_date``.
        value_per_current_model_attributed_conversion (float):
            The value of current model attributed conversions divided by
            the number of the conversions. This only includes conversion
            actions which include_in_conversions_metric attribute is set
            to true. If you use conversion-based bidding, your bid
            strategies will optimize for these conversions.

            This field is a member of `oneof`_ ``_value_per_current_model_attributed_conversion``.
        video_quartile_p100_rate (float):
            Percentage of impressions where the viewer
            watched all of your video.

            This field is a member of `oneof`_ ``_video_quartile_p100_rate``.
        video_quartile_p25_rate (float):
            Percentage of impressions where the viewer
            watched 25% of your video.

            This field is a member of `oneof`_ ``_video_quartile_p25_rate``.
        video_quartile_p50_rate (float):
            Percentage of impressions where the viewer
            watched 50% of your video.

            This field is a member of `oneof`_ ``_video_quartile_p50_rate``.
        video_quartile_p75_rate (float):
            Percentage of impressions where the viewer
            watched 75% of your video.

            This field is a member of `oneof`_ ``_video_quartile_p75_rate``.
        video_view_rate (float):
            The number of views your TrueView video ad
            receives divided by its number of impressions,
            including thumbnail impressions for TrueView
            in-display ads.

            This field is a member of `oneof`_ ``_video_view_rate``.
        video_views (int):
            The number of times your video ads were
            viewed.

            This field is a member of `oneof`_ ``_video_views``.
        view_through_conversions (int):
            The total number of view-through conversions.
            These happen when a customer sees an image or
            rich media ad, then later completes a conversion
            on your site without interacting with (e.g.,
            clicking on) another ad.

            This field is a member of `oneof`_ ``_view_through_conversions``.
        sk_ad_network_conversions (int):
            The number of iOS Store Kit Ad Network
            conversions.
    """

    absolute_top_impression_percentage = proto.Field(
        proto.DOUBLE,
        number=183,
        optional=True,
    )
    active_view_cpm = proto.Field(
        proto.DOUBLE,
        number=184,
        optional=True,
    )
    active_view_ctr = proto.Field(
        proto.DOUBLE,
        number=185,
        optional=True,
    )
    active_view_impressions = proto.Field(
        proto.INT64,
        number=186,
        optional=True,
    )
    active_view_measurability = proto.Field(
        proto.DOUBLE,
        number=187,
        optional=True,
    )
    active_view_measurable_cost_micros = proto.Field(
        proto.INT64,
        number=188,
        optional=True,
    )
    active_view_measurable_impressions = proto.Field(
        proto.INT64,
        number=189,
        optional=True,
    )
    active_view_viewability = proto.Field(
        proto.DOUBLE,
        number=190,
        optional=True,
    )
    all_conversions_from_interactions_rate = proto.Field(
        proto.DOUBLE,
        number=191,
        optional=True,
    )
    all_conversions_value = proto.Field(
        proto.DOUBLE,
        number=192,
        optional=True,
    )
    all_conversions_value_by_conversion_date = proto.Field(
        proto.DOUBLE,
        number=240,
    )
    all_conversions = proto.Field(
        proto.DOUBLE,
        number=193,
        optional=True,
    )
    all_conversions_by_conversion_date = proto.Field(
        proto.DOUBLE,
        number=241,
    )
    all_conversions_value_per_cost = proto.Field(
        proto.DOUBLE,
        number=194,
        optional=True,
    )
    all_conversions_from_click_to_call = proto.Field(
        proto.DOUBLE,
        number=195,
        optional=True,
    )
    all_conversions_from_directions = proto.Field(
        proto.DOUBLE,
        number=196,
        optional=True,
    )
    all_conversions_from_interactions_value_per_interaction = proto.Field(
        proto.DOUBLE,
        number=197,
        optional=True,
    )
    all_conversions_from_menu = proto.Field(
        proto.DOUBLE,
        number=198,
        optional=True,
    )
    all_conversions_from_order = proto.Field(
        proto.DOUBLE,
        number=199,
        optional=True,
    )
    all_conversions_from_other_engagement = proto.Field(
        proto.DOUBLE,
        number=200,
        optional=True,
    )
    all_conversions_from_store_visit = proto.Field(
        proto.DOUBLE,
        number=201,
        optional=True,
    )
    all_conversions_from_store_website = proto.Field(
        proto.DOUBLE,
        number=202,
        optional=True,
    )
    auction_insight_search_absolute_top_impression_percentage = proto.Field(
        proto.DOUBLE,
        number=258,
        optional=True,
    )
    auction_insight_search_impression_share = proto.Field(
        proto.DOUBLE,
        number=259,
        optional=True,
    )
    auction_insight_search_outranking_share = proto.Field(
        proto.DOUBLE,
        number=260,
        optional=True,
    )
    auction_insight_search_overlap_rate = proto.Field(
        proto.DOUBLE,
        number=261,
        optional=True,
    )
    auction_insight_search_position_above_rate = proto.Field(
        proto.DOUBLE,
        number=262,
        optional=True,
    )
    auction_insight_search_top_impression_percentage = proto.Field(
        proto.DOUBLE,
        number=263,
        optional=True,
    )
    average_cost = proto.Field(
        proto.DOUBLE,
        number=203,
        optional=True,
    )
    average_cpc = proto.Field(
        proto.DOUBLE,
        number=204,
        optional=True,
    )
    average_cpe = proto.Field(
        proto.DOUBLE,
        number=205,
        optional=True,
    )
    average_cpm = proto.Field(
        proto.DOUBLE,
        number=206,
        optional=True,
    )
    average_cpv = proto.Field(
        proto.DOUBLE,
        number=207,
        optional=True,
    )
    average_page_views = proto.Field(
        proto.DOUBLE,
        number=208,
        optional=True,
    )
    average_time_on_site = proto.Field(
        proto.DOUBLE,
        number=209,
        optional=True,
    )
    benchmark_average_max_cpc = proto.Field(
        proto.DOUBLE,
        number=210,
        optional=True,
    )
    biddable_app_install_conversions = proto.Field(
        proto.DOUBLE,
        number=254,
        optional=True,
    )
    biddable_app_post_install_conversions = proto.Field(
        proto.DOUBLE,
        number=255,
        optional=True,
    )
    benchmark_ctr = proto.Field(
        proto.DOUBLE,
        number=211,
        optional=True,
    )
    bounce_rate = proto.Field(
        proto.DOUBLE,
        number=212,
        optional=True,
    )
    clicks = proto.Field(
        proto.INT64,
        number=131,
        optional=True,
    )
    combined_clicks = proto.Field(
        proto.INT64,
        number=156,
        optional=True,
    )
    combined_clicks_per_query = proto.Field(
        proto.DOUBLE,
        number=157,
        optional=True,
    )
    combined_queries = proto.Field(
        proto.INT64,
        number=158,
        optional=True,
    )
    content_budget_lost_impression_share = proto.Field(
        proto.DOUBLE,
        number=159,
        optional=True,
    )
    content_impression_share = proto.Field(
        proto.DOUBLE,
        number=160,
        optional=True,
    )
    conversion_last_received_request_date_time = proto.Field(
        proto.STRING,
        number=161,
        optional=True,
    )
    conversion_last_conversion_date = proto.Field(
        proto.STRING,
        number=162,
        optional=True,
    )
    content_rank_lost_impression_share = proto.Field(
        proto.DOUBLE,
        number=163,
        optional=True,
    )
    conversions_from_interactions_rate = proto.Field(
        proto.DOUBLE,
        number=164,
        optional=True,
    )
    conversions_value = proto.Field(
        proto.DOUBLE,
        number=165,
        optional=True,
    )
    conversions_value_by_conversion_date = proto.Field(
        proto.DOUBLE,
        number=242,
    )
    conversions_value_per_cost = proto.Field(
        proto.DOUBLE,
        number=166,
        optional=True,
    )
    conversions_from_interactions_value_per_interaction = proto.Field(
        proto.DOUBLE,
        number=167,
        optional=True,
    )
    conversions = proto.Field(
        proto.DOUBLE,
        number=168,
        optional=True,
    )
    conversions_by_conversion_date = proto.Field(
        proto.DOUBLE,
        number=243,
    )
    cost_micros = proto.Field(
        proto.INT64,
        number=169,
        optional=True,
    )
    cost_per_all_conversions = proto.Field(
        proto.DOUBLE,
        number=170,
        optional=True,
    )
    cost_per_conversion = proto.Field(
        proto.DOUBLE,
        number=171,
        optional=True,
    )
    cost_per_current_model_attributed_conversion = proto.Field(
        proto.DOUBLE,
        number=172,
        optional=True,
    )
    cross_device_conversions = proto.Field(
        proto.DOUBLE,
        number=173,
        optional=True,
    )
    ctr = proto.Field(
        proto.DOUBLE,
        number=174,
        optional=True,
    )
    current_model_attributed_conversions = proto.Field(
        proto.DOUBLE,
        number=175,
        optional=True,
    )
    current_model_attributed_conversions_from_interactions_rate = proto.Field(
        proto.DOUBLE,
        number=176,
        optional=True,
    )
    current_model_attributed_conversions_from_interactions_value_per_interaction = proto.Field(
        proto.DOUBLE,
        number=177,
        optional=True,
    )
    current_model_attributed_conversions_value = proto.Field(
        proto.DOUBLE,
        number=178,
        optional=True,
    )
    current_model_attributed_conversions_value_per_cost = proto.Field(
        proto.DOUBLE,
        number=179,
        optional=True,
    )
    engagement_rate = proto.Field(
        proto.DOUBLE,
        number=180,
        optional=True,
    )
    engagements = proto.Field(
        proto.INT64,
        number=181,
        optional=True,
    )
    hotel_average_lead_value_micros = proto.Field(
        proto.DOUBLE,
        number=213,
        optional=True,
    )
    hotel_commission_rate_micros = proto.Field(
        proto.INT64,
        number=256,
        optional=True,
    )
    hotel_expected_commission_cost = proto.Field(
        proto.DOUBLE,
        number=257,
        optional=True,
    )
    hotel_price_difference_percentage = proto.Field(
        proto.DOUBLE,
        number=214,
        optional=True,
    )
    hotel_eligible_impressions = proto.Field(
        proto.INT64,
        number=215,
        optional=True,
    )
    historical_creative_quality_score = proto.Field(
        proto.ENUM,
        number=80,
        enum=quality_score_bucket.QualityScoreBucketEnum.QualityScoreBucket,
    )
    historical_landing_page_quality_score = proto.Field(
        proto.ENUM,
        number=81,
        enum=quality_score_bucket.QualityScoreBucketEnum.QualityScoreBucket,
    )
    historical_quality_score = proto.Field(
        proto.INT64,
        number=216,
        optional=True,
    )
    historical_search_predicted_ctr = proto.Field(
        proto.ENUM,
        number=83,
        enum=quality_score_bucket.QualityScoreBucketEnum.QualityScoreBucket,
    )
    gmail_forwards = proto.Field(
        proto.INT64,
        number=217,
        optional=True,
    )
    gmail_saves = proto.Field(
        proto.INT64,
        number=218,
        optional=True,
    )
    gmail_secondary_clicks = proto.Field(
        proto.INT64,
        number=219,
        optional=True,
    )
    impressions_from_store_reach = proto.Field(
        proto.INT64,
        number=220,
        optional=True,
    )
    impressions = proto.Field(
        proto.INT64,
        number=221,
        optional=True,
    )
    interaction_rate = proto.Field(
        proto.DOUBLE,
        number=222,
        optional=True,
    )
    interactions = proto.Field(
        proto.INT64,
        number=223,
        optional=True,
    )
    interaction_event_types = proto.RepeatedField(
        proto.ENUM,
        number=100,
        enum=interaction_event_type.InteractionEventTypeEnum.InteractionEventType,
    )
    invalid_click_rate = proto.Field(
        proto.DOUBLE,
        number=224,
        optional=True,
    )
    invalid_clicks = proto.Field(
        proto.INT64,
        number=225,
        optional=True,
    )
    message_chats = proto.Field(
        proto.INT64,
        number=226,
        optional=True,
    )
    message_impressions = proto.Field(
        proto.INT64,
        number=227,
        optional=True,
    )
    message_chat_rate = proto.Field(
        proto.DOUBLE,
        number=228,
        optional=True,
    )
    mobile_friendly_clicks_percentage = proto.Field(
        proto.DOUBLE,
        number=229,
        optional=True,
    )
    optimization_score_uplift = proto.Field(
        proto.DOUBLE,
        number=247,
        optional=True,
    )
    optimization_score_url = proto.Field(
        proto.STRING,
        number=248,
        optional=True,
    )
    organic_clicks = proto.Field(
        proto.INT64,
        number=230,
        optional=True,
    )
    organic_clicks_per_query = proto.Field(
        proto.DOUBLE,
        number=231,
        optional=True,
    )
    organic_impressions = proto.Field(
        proto.INT64,
        number=232,
        optional=True,
    )
    organic_impressions_per_query = proto.Field(
        proto.DOUBLE,
        number=233,
        optional=True,
    )
    organic_queries = proto.Field(
        proto.INT64,
        number=234,
        optional=True,
    )
    percent_new_visitors = proto.Field(
        proto.DOUBLE,
        number=235,
        optional=True,
    )
    phone_calls = proto.Field(
        proto.INT64,
        number=236,
        optional=True,
    )
    phone_impressions = proto.Field(
        proto.INT64,
        number=237,
        optional=True,
    )
    phone_through_rate = proto.Field(
        proto.DOUBLE,
        number=238,
        optional=True,
    )
    relative_ctr = proto.Field(
        proto.DOUBLE,
        number=239,
        optional=True,
    )
    search_absolute_top_impression_share = proto.Field(
        proto.DOUBLE,
        number=136,
        optional=True,
    )
    search_budget_lost_absolute_top_impression_share = proto.Field(
        proto.DOUBLE,
        number=137,
        optional=True,
    )
    search_budget_lost_impression_share = proto.Field(
        proto.DOUBLE,
        number=138,
        optional=True,
    )
    search_budget_lost_top_impression_share = proto.Field(
        proto.DOUBLE,
        number=139,
        optional=True,
    )
    search_click_share = proto.Field(
        proto.DOUBLE,
        number=140,
        optional=True,
    )
    search_exact_match_impression_share = proto.Field(
        proto.DOUBLE,
        number=141,
        optional=True,
    )
    search_impression_share = proto.Field(
        proto.DOUBLE,
        number=142,
        optional=True,
    )
    search_rank_lost_absolute_top_impression_share = proto.Field(
        proto.DOUBLE,
        number=143,
        optional=True,
    )
    search_rank_lost_impression_share = proto.Field(
        proto.DOUBLE,
        number=144,
        optional=True,
    )
    search_rank_lost_top_impression_share = proto.Field(
        proto.DOUBLE,
        number=145,
        optional=True,
    )
    search_top_impression_share = proto.Field(
        proto.DOUBLE,
        number=146,
        optional=True,
    )
    speed_score = proto.Field(
        proto.INT64,
        number=147,
        optional=True,
    )
    top_impression_percentage = proto.Field(
        proto.DOUBLE,
        number=148,
        optional=True,
    )
    valid_accelerated_mobile_pages_clicks_percentage = proto.Field(
        proto.DOUBLE,
        number=149,
        optional=True,
    )
    value_per_all_conversions = proto.Field(
        proto.DOUBLE,
        number=150,
        optional=True,
    )
    value_per_all_conversions_by_conversion_date = proto.Field(
        proto.DOUBLE,
        number=244,
        optional=True,
    )
    value_per_conversion = proto.Field(
        proto.DOUBLE,
        number=151,
        optional=True,
    )
    value_per_conversions_by_conversion_date = proto.Field(
        proto.DOUBLE,
        number=245,
        optional=True,
    )
    value_per_current_model_attributed_conversion = proto.Field(
        proto.DOUBLE,
        number=152,
        optional=True,
    )
    video_quartile_p100_rate = proto.Field(
        proto.DOUBLE,
        number=132,
        optional=True,
    )
    video_quartile_p25_rate = proto.Field(
        proto.DOUBLE,
        number=133,
        optional=True,
    )
    video_quartile_p50_rate = proto.Field(
        proto.DOUBLE,
        number=134,
        optional=True,
    )
    video_quartile_p75_rate = proto.Field(
        proto.DOUBLE,
        number=135,
        optional=True,
    )
    video_view_rate = proto.Field(
        proto.DOUBLE,
        number=153,
        optional=True,
    )
    video_views = proto.Field(
        proto.INT64,
        number=154,
        optional=True,
    )
    view_through_conversions = proto.Field(
        proto.INT64,
        number=155,
        optional=True,
    )
    sk_ad_network_conversions = proto.Field(
        proto.INT64,
        number=246,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2022 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v11.errors",
    marshal="google.ads.googleads.v11",
    manifest={"CampaignCriterionErrorEnum",},
)


class CampaignCriterionErrorEnum(proto.Message):
    r"""Container for enum describing possible campaign criterion
    errors.

    """

    class CampaignCriterionError(proto.Enum):
        r"""Enum describing possible campaign criterion errors."""
        UNSPECIFIED = 0
        UNKNOWN = 1
        CONCRETE_TYPE_REQUIRED = 2
        INVALID_PLACEMENT_URL = 3
        CANNOT_EXCLUDE_CRITERIA_TYPE = 4
        CANNOT_SET_STATUS_FOR_CRITERIA_TYPE = 5
        CANNOT_SET_STATUS_FOR_EXCLUDED_CRITERIA = 6
        CANNOT_TARGET_AND_EXCLUDE = 7
        TOO_MANY_OPERATIONS = 8
        OPERATOR_NOT_SUPPORTED_FOR_CRITERION_TYPE = 9
        SHOPPING_CAMPAIGN_SALES_COUNTRY_NOT_SUPPORTED_FOR_SALES_CHANNEL = 10
        CANNOT_ADD_EXISTING_FIELD = 11
        CANNOT_UPDATE_NEGATIVE_CRITERION = 12
        CANNOT_SET_NEGATIVE_KEYWORD_THEME_CONSTANT_CRITERION = 13
        INVALID_KEYWORD_THEME_CONSTANT = 14
        MISSING_KEYWORD_THEME_CONSTANT_OR_FREE_FORM_KEYWORD_THEME = 15
        CANNOT_TARGET_BOTH_PROXIMITY_AND_LOCATION_CRITERIA_FOR_SMART_CAMPAIGN = (
            16
        )
        CANNOT_TARGET_MULTIPLE_PROXIMITY_CRITERIA_FOR_SMART_CAMPAIGN = 17


__all__ = tuple(sorted(__protobuf__.manifest))
